package cn.com.qmth.dp.examcloud.oe.entity.marking;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Index;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

/**
 * 评卷项 Created by zhengmin on 2016/12/12.
 */
@Entity
@Table(name = "ec_m_mark_item", indexes = {
        @Index(name = "IDX_M_MARK_ITEM_001", columnList = "workId,basePaperId,questionId,mainNumber,subNumber,orders", unique = false) })
public class MarkItemEntity extends IdEntity implements Comparable {

    enum ScoringType {
        /**
         * 根据给分间隔
         */
        INTERVAL,
        /**
         * 人工指定给分列表
         */
        MANUAL
    }

    private static final long serialVersionUID = -3730733150694463597L;

    @NotNull
    private String basePaperId;

    @NotNull
    private Long workId;

    private Long markRangeId;

    /**
     * 大题号
     */
    private int mainNumber;

    /**
     * 小题号
     */
    private int subNumber;

    /**
     * 全试卷范围内题号顺序
     */
    private int orders;

    /**
     * 分值
     */
    private double maxScore;

    /**
     * 给分方式
     */
    @Enumerated(EnumType.STRING)
    private ScoringType scoringType;

    /**
     * 给分间隔
     */
    private Double scoreInterval;

    /**
     * 给分列表
     */
    private String socreList;

    /**
     * 题Id
     */
    private String questionId;

    public MarkItemEntity() {
    }

    public MarkItemEntity(String basePaperId, int mainNumber, int subNumber, double maxScore, Double scoreInterval,
            int orders, String questionId) {
        this.basePaperId = basePaperId;
        this.mainNumber = mainNumber;
        this.subNumber = subNumber;
        this.maxScore = maxScore;
        this.scoreInterval = scoreInterval == null ? 1 : scoreInterval;
        this.scoringType = ScoringType.INTERVAL;
        this.orders = orders;
        this.questionId = questionId;
    }

    public MarkItemEntity(String basePaperId, int mainNumber, int subNumber, double maxScore, String socreList) {
        this.basePaperId = basePaperId;
        this.mainNumber = mainNumber;
        this.subNumber = subNumber;
        this.maxScore = maxScore;
        this.socreList = socreList == null ? String.valueOf(this.maxScore) : socreList;
        this.scoringType = ScoringType.MANUAL;
    }

    public Long getWorkId() {
        return workId;
    }

    public void setWorkId(Long workId) {
        this.workId = workId;
    }

    public int getMainNumber() {
        return mainNumber;
    }

    public void setMainNumber(int mainNumber) {
        this.mainNumber = mainNumber;
    }

    public int getSubNumber() {
        return subNumber;
    }

    public void setSubNumber(int subNumber) {
        this.subNumber = subNumber;
    }

    public double getMaxScore() {
        return maxScore;
    }

    public void setMaxScore(double maxScore) {
        this.maxScore = maxScore;
    }

    public ScoringType getScoringType() {
        return scoringType;
    }

    public void setScoringType(ScoringType scoringType) {
        this.scoringType = scoringType;
    }

    public Double getScoreInterval() {
        return scoreInterval;
    }

    public void setScoreInterval(Double scoreInterval) {
        this.scoreInterval = scoreInterval;
    }

    public String getSocreList() {
        return socreList;
    }

    public void setSocreList(String socreList) {
        this.socreList = socreList;
    }

    @NotNull
    public String getBasePaperId() {
        return basePaperId;
    }

    public void setBasePaperId(@NotNull String basePaperId) {
        this.basePaperId = basePaperId;
    }

    public Long getMarkRangeId() {
        return markRangeId;
    }

    public void setMarkRangeId(Long markRangeId) {
        this.markRangeId = markRangeId;
    }

    public int getOrders() {
        return orders;
    }

    public void setOrders(int orders) {
        this.orders = orders;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;

        MarkItemEntity markItem = (MarkItemEntity) o;

        if (getMainNumber() != markItem.getMainNumber())
            return false;
        if (getSubNumber() != markItem.getSubNumber())
            return false;
        if (getOrders() != markItem.getOrders())
            return false;
        if (getBasePaperId() != null ? !getBasePaperId().equals(markItem.getBasePaperId())
                : markItem.getBasePaperId() != null)
            return false;
        return getWorkId() != null ? getWorkId().equals(markItem.getWorkId()) : markItem.getWorkId() == null;

    }

    @Override
    public int hashCode() {
        int result = getBasePaperId() != null ? getBasePaperId().hashCode() : 0;
        result = 31 * result + (getWorkId() != null ? getWorkId().hashCode() : 0);
        result = 31 * result + getMainNumber();
        result = 31 * result + getSubNumber();
        result = 31 * result + getOrders();
        return result;
    }

    @Override
    public int compareTo(Object o) {
        MarkItemEntity markItem = (MarkItemEntity) o;
        if (markItem.getOrders() < this.getOrders()) {
            return 1;
        } else if (markItem.getOrders() > this.getOrders()) {
            return -1;
        } else {
            return 0;
        }
    }

    public String getQuestionId() {
        return questionId;
    }

    public void setQuestionId(String questionId) {
        this.questionId = questionId;
    }
}
